/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "AnalogueRepeaterConfigAckSet.h"

const unsigned int ACK_WIDTH1 = 300U;
const unsigned int ACK_WIDTH2 = 75U;
const unsigned int ACK_WIDTH3 = 200U;

const unsigned int BORDER_SIZE = 5U;

CAnalogueRepeaterConfigAckSet::CAnalogueRepeaterConfigAckSet(wxWindow* parent, int id, const wxString& title, const wxString& radioAck, const wxString& externalAck, unsigned int speed, unsigned int freq, wxFloat32 level, unsigned int delay, unsigned int minimum) :
wxPanel(parent, id),
m_title(title),
m_radioAck(NULL),
m_externalAck(NULL),
m_speed(NULL),
m_freq(NULL),
m_level(NULL),
m_delay(NULL),
m_minimum(NULL)
{
	wxFlexGridSizer* sizer = new wxFlexGridSizer(2);

	wxStaticText* radioAckLabel = new wxStaticText(this, -1, _("Radio Ack"));
	sizer->Add(radioAckLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_radioAck = new CCWTextCtrl(this, -1, radioAck, wxDefaultPosition, wxSize(ACK_WIDTH3, -1));
	m_radioAck->SetMaxLength(20U);
	sizer->Add(m_radioAck, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* externalAckLabel = new wxStaticText(this, -1, _("External Ack"));
	sizer->Add(externalAckLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_externalAck = new CCWTextCtrl(this, -1, externalAck, wxDefaultPosition, wxSize(ACK_WIDTH3, -1));
	m_externalAck->SetMaxLength(20U);
	sizer->Add(m_externalAck, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* speedLabel = new wxStaticText(this, -1, _("Speed (WPM)"));
	sizer->Add(speedLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_speed = new wxSlider(this, -1, speed, 8, 25, wxDefaultPosition, wxSize(ACK_WIDTH1, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_speed, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* freqLabel = new wxStaticText(this, -1, _("Frequency (Hz)"));
	sizer->Add(freqLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_freq = new wxSlider(this, -1, freq, 500, 2000, wxDefaultPosition, wxSize(ACK_WIDTH1, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_freq, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* levelLabel = new wxStaticText(this, -1, _("Level (%)"));
	sizer->Add(levelLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_level = new wxSlider(this, -1, int(level * 100.0F + 0.5F), 0, 100, wxDefaultPosition, wxSize(ACK_WIDTH1, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_level, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* delayLabel = new wxStaticText(this, -1, _("Delay (s)"));
	sizer->Add(delayLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_delay = new wxSlider(this, -1, delay, 1, 3, wxDefaultPosition, wxSize(ACK_WIDTH1, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_delay, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	wxStaticText* minimumLabel = new wxStaticText(this, -1, _("Minimum (s)"));
	sizer->Add(minimumLabel, 0, wxALL | wxALIGN_RIGHT, BORDER_SIZE);

	m_minimum = new wxSlider(this, -1, minimum, 0, 10, wxDefaultPosition, wxSize(ACK_WIDTH1, -1), wxSL_HORIZONTAL | wxSL_LABELS);
	sizer->Add(m_minimum, 0, wxALL | wxALIGN_LEFT, BORDER_SIZE);

	SetAutoLayout(true);

	sizer->Fit(this);
	sizer->SetSizeHints(this);

	SetSizer(sizer);
}


CAnalogueRepeaterConfigAckSet::~CAnalogueRepeaterConfigAckSet()
{
}

bool CAnalogueRepeaterConfigAckSet::Validate()
{
	return true;
}

wxString CAnalogueRepeaterConfigAckSet::getRadioAck() const
{
	return m_radioAck->GetValue();
}

wxString CAnalogueRepeaterConfigAckSet::getExternalAck() const
{
	return m_externalAck->GetValue();
}

unsigned int CAnalogueRepeaterConfigAckSet::getSpeed() const
{
	return m_speed->GetValue();
}

unsigned int CAnalogueRepeaterConfigAckSet::getFreq() const
{
	return m_freq->GetValue();
}

wxFloat32 CAnalogueRepeaterConfigAckSet::getLevel() const
{
	return wxFloat32(m_level->GetValue()) / 100.0F;
}

unsigned int CAnalogueRepeaterConfigAckSet::getDelay() const
{
	return m_delay->GetValue();
}

unsigned int CAnalogueRepeaterConfigAckSet::getMinimum() const
{
	return m_minimum->GetValue();
}
