/*
 *	Copyright (C) 2009 by Jonathan Naylor, G4KLX
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 */

#include "NetworkDelay.h"

CNetworkDelay::CNetworkDelay(unsigned int delay) :
m_delay(2U * delay),
m_in(delay),
m_out(0U),
m_bufList(NULL)
{
	wxASSERT(delay > 0U);

	m_bufList = new SNetworkDelayObject[m_delay];

	for (unsigned int i = 0U; i < m_delay; i++) {
		m_bufList[i].type   = NETWORK_NONE;
		m_bufList[i].header = NULL;
		m_bufList[i].length = 0U;
		m_bufList[i].end    = false;
	}
}

CNetworkDelay::~CNetworkDelay()
{
	delete[] m_bufList;
}

NETWORK_TYPE CNetworkDelay::delay()
{
	m_bufList[m_in].type = NETWORK_NONE;

	m_in++;
	if (m_in >= m_delay)
		m_in = 0U;

	// If not interesting data then advance the out pointer
	if (m_bufList[m_out].type == NETWORK_NONE) {
		m_out++;
		if (m_out >= m_delay)
			m_out = 0U;

		return NETWORK_NONE;
	}

	return m_bufList[m_out].type;
}

NETWORK_TYPE CNetworkDelay::delay(NETWORK_TYPE type, CHeaderData* header)
{
	wxASSERT(header != NULL);

	m_bufList[m_in].type   = type;
	m_bufList[m_in].header = header;

	m_in++;
	if (m_in >= m_delay)
		m_in = 0U;

	// If not interesting data then advance the out pointer
	if (m_bufList[m_out].type == NETWORK_NONE) {
		m_out++;
		if (m_out >= m_delay)
			m_out = 0U;

		return NETWORK_NONE;
	}

	return m_bufList[m_out].type;
}

NETWORK_TYPE CNetworkDelay::delay(NETWORK_TYPE type, const unsigned char* data, unsigned int length, bool end)
{
	wxASSERT(data != NULL);
	wxASSERT(length > 0U);

	if (length > DV_FRAME_MAX_LENGTH_BYTES)
		length = DV_FRAME_MAX_LENGTH_BYTES;

	::memcpy(m_bufList[m_in].data, data, length * sizeof(unsigned char));

	m_bufList[m_in].type   = type;
	m_bufList[m_in].length = length;
	m_bufList[m_in].end    = end;

	m_in++;
	if (m_in >= m_delay)
		m_in = 0U;

	// If not interesting data then advance the out pointer
	if (m_bufList[m_out].type == NETWORK_NONE) {
		m_out++;
		if (m_out >= m_delay)
			m_out = 0U;

		return NETWORK_NONE;
	}

	return m_bufList[m_out].type;
}

CHeaderData* CNetworkDelay::readHeader()
{
	if (m_bufList[m_out].type != NETWORK_USER_HEADER && m_bufList[m_out].type != NETWORK_STATUS_HEADER)
		return NULL;

	CHeaderData* header = m_bufList[m_out].header;

	m_bufList[m_out].header = NULL;
	m_bufList[m_out].type   = NETWORK_NONE;

	m_out++;
	if (m_out >= m_delay)
		m_out = 0U;

	return header;
}

unsigned int CNetworkDelay::readData(unsigned char* data, unsigned int length, bool& end)
{
	if (m_bufList[m_out].type != NETWORK_USER_DATA && m_bufList[m_out].type != NETWORK_STATUS_DATA)
		return 0U;

	unsigned int len = m_bufList[m_out].length;
	if (len > length)
		len = length;

	::memcpy(data, m_bufList[m_out].data, len * sizeof(unsigned char));

	end = m_bufList[m_out].end;

	m_bufList[m_out].length = 0U;
	m_bufList[m_out].end    = false;
	m_bufList[m_out].type   = NETWORK_NONE;

	m_out++;
	if (m_out >= m_delay)
		m_out = 0U;

	return len;
}
