/*
 *	Copyright (C) 2009 by Jonathan Naylor, G4KLX
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 */

#include "SlowDataDecoder.h"
#include "DStarDefines.h"

const unsigned char SCRAMBLER_BYTE1 = 0x70;
const unsigned char SCRAMBLER_BYTE2 = 0x4F;
const unsigned char SCRAMBLER_BYTE3 = 0x93;

const unsigned char SLOW_DATA_TYPE_MASK    = 0xF0;
const unsigned char SLOW_DATA_TYPE_HEADER  = 0x50;
const unsigned char SLOW_DATA_LENGTH_MASK  = 0x0F;

const unsigned int MESSAGE_LENGTH = 20U;

CSlowDataDecoder::CSlowDataDecoder() :
m_buffer(NULL),
m_state(SDD_FIRST),
m_headerData(NULL),
m_headerPtr(0U),
m_header(NULL)
{
	m_buffer     = new unsigned char[6U];

	m_headerData = new unsigned char[RADIO_HEADER_LENGTH_BYTES * 2U];
}

CSlowDataDecoder::~CSlowDataDecoder()
{
	delete[] m_buffer;

	delete[] m_headerData;

	delete m_header;
}

void CSlowDataDecoder::addData(const unsigned char* data)
{
	wxASSERT(data != NULL);

	switch (m_state) {
		case SDD_FIRST:
			m_buffer[0] = data[0] ^ SCRAMBLER_BYTE1;
			m_buffer[1] = data[1] ^ SCRAMBLER_BYTE2;
			m_buffer[2] = data[2] ^ SCRAMBLER_BYTE3;
			m_state = SDD_SECOND;
			return;

		case SDD_SECOND:
			m_buffer[3] = data[0] ^ SCRAMBLER_BYTE1;
			m_buffer[4] = data[1] ^ SCRAMBLER_BYTE2;
			m_buffer[5] = data[2] ^ SCRAMBLER_BYTE3;
			m_state = SDD_FIRST;
			break;
	}			

	if ((m_buffer[0] & SLOW_DATA_TYPE_MASK) == SLOW_DATA_TYPE_HEADER)
		processHeader();
}

CHeaderData* CSlowDataDecoder::getHeaderData()
{
	if (m_header == NULL)
		return NULL;

	CHeaderData* temp = new CHeaderData(*m_header);

	reset();

	return temp;
}

void CSlowDataDecoder::sync()
{
	m_state = SDD_FIRST;
}

void CSlowDataDecoder::reset()
{
	m_headerPtr = 0U;

	m_state = SDD_FIRST;

	delete m_header;
	m_header = NULL;
}

void CSlowDataDecoder::processHeader()
{
	// Do we have a complete and valid header already?
	if (m_header != NULL)
		return;

	unsigned int length = m_buffer[0] & SLOW_DATA_LENGTH_MASK;
	if (length > 5U)
		return;

	for (unsigned int i = 1U; i <= length; i++, m_headerPtr++)
		m_headerData[m_headerPtr] = m_buffer[i];

	// Do we have a complete header?
	if (m_headerPtr < RADIO_HEADER_LENGTH_BYTES)
		return;

	// Something has been corrupted, discard
	if (m_headerPtr != RADIO_HEADER_LENGTH_BYTES) {
		reset();
		return;
	}

	m_header = new CHeaderData(m_headerData, RADIO_HEADER_LENGTH_BYTES, true);

	if (!m_header->isValid())
		reset();
}
