/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "DStarRepeaterFrame.h"
#include "DStarRepeaterApp.h"
#include "WAVFileReader.h"
#include "DStarDefines.h"
#include "LogEvent.h"
#include "Version.h"

#include <wx/aboutdlg.h>

DEFINE_EVENT_TYPE(LOG_EVENT)

enum {
	Menu_View_Updates = 6000,

	Menu_Action_Shutdown,
	Menu_Action_Startup,
	Menu_Action_Command1,
	Menu_Action_Command2,

	Menu_Output_1,
	Menu_Output_2,
	Menu_Output_3,
	Menu_Output_4
};

BEGIN_EVENT_TABLE(CDStarRepeaterFrame, wxFrame)
#if defined(__WXDEBUG__)
	EVT_MENU(wxID_OPEN, CDStarRepeaterFrame::onOpen)
#endif
	EVT_MENU(wxID_EXIT, CDStarRepeaterFrame::onQuit)
	EVT_MENU(Menu_View_Updates, CDStarRepeaterFrame::onUpdates)
	EVT_MENU(wxID_ABOUT, CDStarRepeaterFrame::onAbout)

	EVT_MENU(Menu_Action_Startup,   CDStarRepeaterFrame::onActions)
	EVT_MENU(Menu_Action_Shutdown,  CDStarRepeaterFrame::onActions)
	EVT_MENU(Menu_Action_Command1,  CDStarRepeaterFrame::onActions)
	EVT_MENU(Menu_Action_Command2,  CDStarRepeaterFrame::onActions)

	EVT_MENU(Menu_Output_1, CDStarRepeaterFrame::onOutputs)
	EVT_MENU(Menu_Output_2, CDStarRepeaterFrame::onOutputs)
	EVT_MENU(Menu_Output_3, CDStarRepeaterFrame::onOutputs)
	EVT_MENU(Menu_Output_4, CDStarRepeaterFrame::onOutputs)

	EVT_CLOSE(CDStarRepeaterFrame::onClose)

	EVT_TIMER(-1, CDStarRepeaterFrame::onTimer)

	EVT_CUSTOM(LOG_EVENT, wxID_ANY, CDStarRepeaterFrame::onLog)
END_EVENT_TABLE()

CDStarRepeaterFrame::CDStarRepeaterFrame(const wxString& title) :
wxFrame(NULL, -1, title),
m_timer(this),
m_panel(NULL),
m_outputMenu(NULL),
#if defined(__WXDEBUG__)
m_updates(true)
#else
m_updates(false)
#endif
{
	SetMenuBar(createMenuBar());

	wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);

	wxPanel* panel = new wxPanel(this);

	wxBoxSizer* panelSizer = new wxBoxSizer(wxVERTICAL);

	m_panel = new CDStarRepeaterPanel(panel);
	panelSizer->Add(m_panel);

	panel->SetSizer(panelSizer);
	panelSizer->SetSizeHints(panel);

	mainSizer->Add(panel);

	SetSizer(mainSizer);
	mainSizer->SetSizeHints(this);

	m_timer.Start(1000);
}

CDStarRepeaterFrame::~CDStarRepeaterFrame()
{
}

wxMenuBar* CDStarRepeaterFrame::createMenuBar()
{
	wxMenu* fileMenu = new wxMenu();
#if defined(__WXDEBUG__)
	fileMenu->Append(wxID_OPEN, _("Open...\tCtrl+N"));
	fileMenu->AppendSeparator();
#endif
	fileMenu->Append(wxID_EXIT, _("Exit"));

	wxMenu* viewMenu = new wxMenu();
	viewMenu->AppendCheckItem(Menu_View_Updates, _("GUI Updates"));
	viewMenu->Check(Menu_View_Updates, m_updates);

	wxMenu* actionMenu = new wxMenu();
	actionMenu->Append(Menu_Action_Shutdown, _("Shutdown"));
	actionMenu->Append(Menu_Action_Startup,  _("Startup"));
	actionMenu->Append(Menu_Action_Command1, _("Command 1"));
	actionMenu->Append(Menu_Action_Command2, _("Command 2"));

	m_outputMenu = new wxMenu();
	m_outputMenu->AppendCheckItem(Menu_Output_1, _("Output 1"));
	m_outputMenu->AppendCheckItem(Menu_Output_2, _("Output 2"));
	m_outputMenu->AppendCheckItem(Menu_Output_3, _("Output 3"));
	m_outputMenu->AppendCheckItem(Menu_Output_4, _("Output 4"));

	wxMenu* helpMenu = new wxMenu();
	helpMenu->Append(wxID_ABOUT, _("About D-Star Repeater"));

	wxMenuBar* menuBar = new wxMenuBar();
	menuBar->Append(fileMenu,     _("File"));
	menuBar->Append(viewMenu,     _("View"));
	menuBar->Append(actionMenu,   _("Action"));
	menuBar->Append(m_outputMenu, _("Outputs"));
	menuBar->Append(helpMenu,     _("Help"));

	return menuBar;
}

#if defined(__WXDEBUG__)
void CDStarRepeaterFrame::onOpen(wxCommandEvent& event)
{
	wxFileDialog dialog(this, _("Select a WAV File"), wxEmptyString, wxEmptyString, _("WAV file (*.wav)|*.wav|All files (*.*)|*.*"));
	if (dialog.ShowModal() != wxID_OK)
		return;

	wxString fileName = dialog.GetPath();

	CWAVFileReader* reader = new CWAVFileReader(fileName, DSTAR_RADIO_BLOCK_SIZE);
	bool res = reader->open();
	if (!res) {
		wxMessageDialog dialog(this, _("Cannot find the file, or the file is corrupt."), _("D-Star Repeater Error"), wxICON_ERROR);
		dialog.ShowModal();
		delete reader;
		return;
	}

	if (reader->getSampleRate() != DSTAR_RADIO_SAMPLE_RATE) {
		::wxLogError(wxT("File %s has an invalid sample rate of %u"), fileName.c_str(), reader->getSampleRate());
		wxMessageDialog dialog(this, _("The sound file has an invalid sample rate."), _("D-Star Repeater Error"), wxICON_ERROR);
		dialog.ShowModal();
		reader->close();
		delete reader;
		return;
	}

	if (reader->getChannels() != 1U) {
		::wxLogError(wxT("File %s has too many channels - %u"), fileName.c_str(), reader->getChannels());
		wxMessageDialog dialog(this, _("The sound file has too many channels."), _("D-Star Repeater Error"), wxICON_ERROR);
		dialog.ShowModal();
		reader->close();
		delete reader;
		return;
	}

	::wxGetApp().setReader(reader);
}
#endif

void CDStarRepeaterFrame::onQuit(wxCommandEvent& event)
{
	Close(false);
}

void CDStarRepeaterFrame::onClose(wxCloseEvent& event)
{
	Destroy();
}

void CDStarRepeaterFrame::onActions(wxCommandEvent& event)
{
	int id = event.GetId();

	switch (id) {
		case Menu_Action_Shutdown:
			::wxGetApp().shutdown();
			break;
		case Menu_Action_Startup:
			::wxGetApp().startup();
			break;
		case Menu_Action_Command1:
			::wxGetApp().command1();
			break;
		case Menu_Action_Command2:
			::wxGetApp().command2();
			break;
	}
}

void CDStarRepeaterFrame::onOutputs(wxCommandEvent& event)
{
	bool out1 = m_outputMenu->IsChecked(Menu_Output_1);
	bool out2 = m_outputMenu->IsChecked(Menu_Output_2);
	bool out3 = m_outputMenu->IsChecked(Menu_Output_3);
	bool out4 = m_outputMenu->IsChecked(Menu_Output_4);

	::wxGetApp().setOutputs(out1, out2, out3, out4);
}

void CDStarRepeaterFrame::onUpdates(wxCommandEvent& event)
{
	m_updates = event.IsChecked();
}

void CDStarRepeaterFrame::onAbout(wxCommandEvent& event)
{
	wxAboutDialogInfo info;
	info.AddDeveloper(wxT("Jonathan Naylor, G4KLX"));
	info.SetCopyright(wxT("(C) 2009,2010 using GPL v2 or later"));
	info.SetName(APPLICATION_NAME);
	info.SetVersion(VERSION);
	info.SetDescription(_("This program allows a computer with a sound card\nand 9600Bd capable radios to act as a D-Star repeater."));

	::wxAboutBox(info);
}

void CDStarRepeaterFrame::onTimer(wxTimerEvent& event)
{
	if (!m_updates)
		return;

	CDStarRepeaterStatusData* status = ::wxGetApp().getStatus();
	if (status == NULL)
		return;

	m_panel->update(*status);

	delete status;
}

void CDStarRepeaterFrame::onLog(wxEvent& event)
{
	CLogEvent& logEvent = dynamic_cast<CLogEvent&>(event);

	wxString text = logEvent.getText();

	m_panel->showLog(text);
}

void CDStarRepeaterFrame::showLog(const wxString& text)
{
	if (!m_updates)
		return;

	CLogEvent event(text, LOG_EVENT);

	AddPendingEvent(event);
}

void CDStarRepeaterFrame::setOutputs(bool out1, bool out2, bool out3, bool out4)
{
	m_outputMenu->Check(Menu_Output_1, out1);
	m_outputMenu->Check(Menu_Output_2, out2);
	m_outputMenu->Check(Menu_Output_3, out3);
	m_outputMenu->Check(Menu_Output_4, out4);
}
