/*
 *   Copyright (C) 2009,2010 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "DStarRepeaterConfigApp.h"
#include "Version.h"

#include <wx/cmdline.h>

IMPLEMENT_APP(CDStarRepeaterConfigApp)

const wxChar* NAME_PARAM = wxT("Repeater Name");


CDStarRepeaterConfigApp::CDStarRepeaterConfigApp() :
wxApp(),
m_name(),
m_frame(NULL),
m_config(NULL)
{
}

CDStarRepeaterConfigApp::~CDStarRepeaterConfigApp()
{
}

bool CDStarRepeaterConfigApp::OnInit()
{
	SetVendorName(VENDOR_NAME);

	if (!wxApp::OnInit())
		return false;

	m_config = new CDStarRepeaterConfig(new wxConfig(APPLICATION_NAME), m_name);

	wxString frameName = APPLICATION_NAME + wxT(" - ");
	if (!m_name.IsEmpty()) {
		frameName.Append(m_name);
		frameName.Append(wxT(" - "));
	}
	frameName.Append(VERSION);

	m_frame = new CDStarRepeaterConfigFrame(frameName);
	m_frame->Show();

	SetTopWindow(m_frame);

	return wxApp::OnInit();
}

int CDStarRepeaterConfigApp::OnExit()
{
	delete m_config;

	return 0;
}

void CDStarRepeaterConfigApp::OnInitCmdLine(wxCmdLineParser& parser)
{
	parser.AddParam(NAME_PARAM, wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL);

	wxApp::OnInitCmdLine(parser);
}

bool CDStarRepeaterConfigApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
	if (!wxApp::OnCmdLineParsed(parser))
		return false;

	if (parser.GetParamCount() > 0U)
		m_name = parser.GetParam(0);

	return true;
}

void CDStarRepeaterConfigApp::getCallsign(wxString& callsign, wxString& gateway, DSTAR_MODE& mode, bool& ack) const
{
	m_config->getCallsign(callsign, gateway, mode, ack);
}

void CDStarRepeaterConfigApp::setCallsign(const wxString& callsign, const wxString& gateway, DSTAR_MODE mode, bool ack) const
{
	m_config->setCallsign(callsign, gateway, mode, ack);
}

void CDStarRepeaterConfigApp::getNetwork(wxString& gatewayAddress, unsigned int& gatewayPort, wxString& localAddress, unsigned int& localPort, unsigned int& networkDelay) const
{
	m_config->getNetwork(gatewayAddress, gatewayPort, localAddress, localPort, networkDelay);
}

void CDStarRepeaterConfigApp::setNetwork(const wxString& gatewayAddress, unsigned int gatewayPort, const wxString& localAddress, unsigned int localPort, unsigned int networkDelay) const
{
	m_config->setNetwork(gatewayAddress, gatewayPort, localAddress, localPort, networkDelay);
}

void CDStarRepeaterConfigApp::getRadio(wxString& readDevice, wxString& writeDevice, wxFloat32& level, bool& rxInvert, bool& txInvert) const
{
	m_config->getRadio(readDevice, writeDevice, level, rxInvert, txInvert);
}

void CDStarRepeaterConfigApp::setRadio(const wxString& readDevice, const wxString& writeDevice, wxFloat32 level, bool rxInvert, bool txInvert) const
{
	m_config->setRadio(readDevice, writeDevice, level, rxInvert, txInvert);
}

void CDStarRepeaterConfigApp::getController(wxString& type, int& pttDelay, bool& pttInvert) const
{
	m_config->getController(type, pttDelay, pttInvert);
}

void CDStarRepeaterConfigApp::setController(const wxString& type, int pttDelay, bool pttInvert) const
{
	m_config->setController(type, pttDelay, pttInvert);
}

void CDStarRepeaterConfigApp::getTimes(unsigned int& timeout, unsigned int& ackTime) const
{
	m_config->getTimes(timeout, ackTime);
}

void CDStarRepeaterConfigApp::setTimes(unsigned int timeout, unsigned int ackTime) const
{
	m_config->setTimes(timeout, ackTime);
}

void CDStarRepeaterConfigApp::getControl(bool& enabled, wxString& rpt1Callsign, wxString& rpt2Callsign, wxString& shutdown, wxString& startup, wxString& command1, wxString& command1Line, wxString& command2, wxString& command2Line, wxString& output1, wxString& output2, wxString& output3, wxString& output4) const
{
	m_config->getControl(enabled, rpt1Callsign, rpt2Callsign, shutdown, startup, command1, command1Line, command2, command2Line, output1, output2, output3, output4);
}

void CDStarRepeaterConfigApp::setControl(bool enabled, const wxString& rpt1Callsign, const wxString& rpt2Callsign, const wxString& shutdown, const wxString& startup, const wxString& command1, const wxString& command1Line, const wxString& command2, const wxString& command2Line, const wxString& output1, const wxString& output2, const wxString& output3, const wxString& output4) const
{
	m_config->setControl(enabled, rpt1Callsign, rpt2Callsign, shutdown, startup, command1, command1Line, command2, command2Line, output1, output2, output3, output3);
}

void CDStarRepeaterConfigApp::getActiveHang(unsigned int& time) const
{
	m_config->getActiveHang(time);
}

void CDStarRepeaterConfigApp::setActiveHang(unsigned int time) const
{
	m_config->setActiveHang(time);
}
