/*
 *   Copyright (C) 2009 by Jonathan Naylor G4KLX
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ParrotControllerConfig.h"

static const wxString  KEY_MODE             = wxT("/mode");
static const wxString  KEY_BEACON_TIME      = wxT("/beaconTime");
static const wxString  KEY_BEACON_FILENAME  = wxT("/beaconFileName");
static const wxString  KEY_TURNAROUND_TIME  = wxT("/turnaroundTime");
static const wxString  KEY_KEEP_FILE        = wxT("/keepFile");
static const wxString  KEY_REPEATER_ADDRESS = wxT("/repeaterAddress");
static const wxString  KEY_REPEATER_PORT    = wxT("/repeaterPort");
static const wxString  KEY_LOCAL_ADDRESS    = wxT("/localAddress");
static const wxString  KEY_LOCAL_PORT       = wxT("/localPort");
static const wxString  KEY_PROTOCOL         = wxT("/protocol");

static const PARROT_MODE DEFAULT_MODE             = PM_PARROT;
static const long        DEFAULT_BEACON_TIME      = 60L;
static const wxString    DEFAULT_BEACON_FILENAME  = wxEmptyString;
static const long        DEFAULT_TURNAROUND_TIME  = 2L;
static const bool        DEFAULT_KEEP_FILE        = false;
static const wxString    DEFAULT_REPEATER_ADDRESS = wxEmptyString;
static const long        DEFAULT_REPEATER_PORT    = 20000L;
static const wxString    DEFAULT_LOCAL_ADDRESS    = wxEmptyString;
static const long        DEFAULT_LOCAL_PORT       = 20000L;


CParrotControllerConfig::CParrotControllerConfig(wxConfigBase* config, const wxString& name) :
m_config(config),
m_name()
{
	wxASSERT(config != NULL);

	if (!name.IsEmpty())
		m_name = wxT("/") + name;
}

CParrotControllerConfig::~CParrotControllerConfig()
{
	delete m_config;
}

void CParrotControllerConfig::getMode(PARROT_MODE& mode, unsigned int& beaconTime, wxString& beaconFileName, unsigned int& turnaroundTime, bool& keepFile) const
{
	wxString modeKey           = m_name + KEY_MODE;
	wxString beaconTimeKey     = m_name + KEY_BEACON_TIME;
	wxString beaconFileNameKey = m_name + KEY_BEACON_FILENAME;
	wxString turnaroundTimeKey = m_name + KEY_TURNAROUND_TIME;
	wxString keepFileKey       = m_name + KEY_KEEP_FILE;

	long temp;
	m_config->Read(modeKey,           &temp,           long(DEFAULT_MODE));
	mode = PARROT_MODE(temp);

	m_config->Read(beaconTimeKey,     &temp,           DEFAULT_BEACON_TIME);
	beaconTime = (unsigned int)temp;

	m_config->Read(beaconFileNameKey, &beaconFileName, DEFAULT_BEACON_FILENAME);

	m_config->Read(turnaroundTimeKey, &temp,           DEFAULT_TURNAROUND_TIME);
	turnaroundTime = (unsigned int)temp;

	m_config->Read(keepFileKey,       &keepFile,       DEFAULT_KEEP_FILE);
}

void CParrotControllerConfig::setMode(PARROT_MODE mode, unsigned int beaconTime, const wxString& beaconFileName, unsigned int turnaroundTime, bool keepFile) const
{
	wxString modeKey           = m_name + KEY_MODE;
	wxString beaconTimeKey     = m_name + KEY_BEACON_TIME;
	wxString beaconFileNameKey = m_name + KEY_BEACON_FILENAME;
	wxString turnaroundTimeKey = m_name + KEY_TURNAROUND_TIME;
	wxString keepFileKey       = m_name + KEY_KEEP_FILE;

	m_config->Write(modeKey,           long(mode));
	m_config->Write(beaconTimeKey,     long(beaconTime));
	m_config->Write(beaconFileNameKey, beaconFileName);
	m_config->Write(turnaroundTimeKey, long(turnaroundTime));
	m_config->Write(keepFileKey,       keepFile);
	m_config->Flush();
}

void CParrotControllerConfig::getNetwork(wxString& repeaterAddress, unsigned int& repeaterPort, wxString& localAddress, unsigned int& localPort) const
{
	wxString repeaterAddressKey = m_name + KEY_REPEATER_ADDRESS;
	wxString repeaterPortKey    = m_name + KEY_REPEATER_PORT;
	wxString localAddressKey    = m_name + KEY_LOCAL_ADDRESS;
	wxString localPortKey       = m_name + KEY_LOCAL_PORT;

	m_config->Read(repeaterAddressKey, &repeaterAddress, DEFAULT_REPEATER_ADDRESS);

	long temp;
	m_config->Read(repeaterPortKey,    &temp,            DEFAULT_REPEATER_PORT);
	repeaterPort = (unsigned int)temp;

	m_config->Read(localAddressKey,    &localAddress,    DEFAULT_LOCAL_ADDRESS);

	m_config->Read(localPortKey,       &temp,            DEFAULT_LOCAL_PORT);
	localPort = (unsigned int)temp;
}

void CParrotControllerConfig::setNetwork(const wxString& repeaterAddress, unsigned int repeaterPort, const wxString& localAddress, unsigned int localPort) const
{
	wxString repeaterAddressKey = m_name + KEY_REPEATER_ADDRESS;
	wxString repeaterPortKey    = m_name + KEY_REPEATER_PORT;
	wxString localAddressKey    = m_name + KEY_LOCAL_ADDRESS;
	wxString localPortKey       = m_name + KEY_LOCAL_PORT;

	m_config->Write(repeaterAddressKey, repeaterAddress);
	m_config->Write(repeaterPortKey,    long(repeaterPort));
	m_config->Write(localAddressKey,    localAddress);
	m_config->Write(localPortKey,       long(localPort));
	m_config->Flush();
}
